package betterwithmods.common.blocks.tile.gen;

import betterwithmods.api.block.IMechanical;
import betterwithmods.api.capabilities.MechanicalCapability;
import betterwithmods.api.tile.IMechanicalPower;
import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.mechanical.BlockMillGenerator;
import betterwithmods.common.blocks.tile.TileBasic;
import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public abstract class TileEntityMillGenerator extends TileBasic implements ITickable, IMechanicalPower {
    //Every generator will take up a single block with no extended bounding box
    public int radius;
    public byte runningState = 0;
    public float speed = 0.0F;
    public float runningSpeed = 0.4F;
    public int overpowerTime = 30;

    public byte dyeIndex = 0;
    public float currentRotation = 0.0F;
    public float previousRotation = 0.0F;
    protected byte waterMod = 1;
    protected EnumFacing facing = null;

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);

        if (tag.func_74764_b("RunningState"))
            runningState = tag.func_74771_c("RunningState");
        if (tag.func_74764_b("CurrentRotation"))
            currentRotation = tag.func_74760_g("CurrentRotation");
        if (tag.func_74764_b("RotationSpeed"))
            previousRotation = tag.func_74760_g("RotationSpeed");
        if (tag.func_74764_b("DyeIndex"))
            dyeIndex = tag.func_74771_c("DyeIndex");
        if (tag.func_74764_b("Facing"))
            facing = EnumFacing.func_82600_a(tag.func_74771_c("Facing"));
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound t = super.func_189515_b(tag);

        t.func_74774_a("RunningState", runningState);
        t.func_74776_a("CurrentRotation", currentRotation);
        t.func_74776_a("RotationSpeed", previousRotation);
        if (facing != null)
            t.func_74774_a("Facing", (byte) facing.ordinal());
        return t;
    }

    public float getCurrentRotation() {
        return this.currentRotation;
    }

    public float getPrevRotation() {
        return this.previousRotation;
    }

    @Override
    public void func_73660_a() {
        if (facing == null) {
            facing = ((BlockMillGenerator) func_145831_w().func_180495_p(field_174879_c).func_177230_c()).getAxleDirectionFromState(func_145831_w().func_180495_p(field_174879_c));
        }
        if (this.runningState != 0) {
            this.previousRotation = (this.runningState > 1 ? this.runningSpeed * 5 : this.runningSpeed) * this.waterMod;//this.currentRotation;
            this.currentRotation += (this.runningState > 1 ? 5 : this.runningState) * this.runningSpeed * this.waterMod;
            if (this.func_145831_w().field_73012_v.nextInt(100) == 0)
                this.func_145831_w().func_184133_a(null, field_174879_c, BWSounds.WOODCREAK, SoundCategory.BLOCKS, 0.75F, func_145831_w().field_73012_v.nextFloat() * 0.25F + 0.25F);
        }
        if (this.currentRotation >= 360.0F)
            this.currentRotation -= 360.0F;
        if (this.func_145831_w().func_82737_E() % 20L == 0L && func_145831_w().func_180495_p(field_174879_c).func_177230_c() instanceof BlockMillGenerator) {
            verifyIntegrity();
            updateSpeed();
            if (this.runningState == 2 && overpowerTime < 1) {
                overpower();
            } else if (this.runningState != 2 && overpowerTime != 30)
                overpowerTime = 30;
            else if (this.runningState == 2 && overpowerTime > 0)
                overpowerTime--;
        }
    }

    public EnumFacing getOrientation() {
        return facing;
    }

    public void setOrientation(EnumFacing facing) {
        if (facing != null) {
            if (facing.func_176743_c() != EnumFacing.AxisDirection.POSITIVE)
                this.facing = facing.func_176734_d();
            else
                this.facing = facing;
        }
    }

    public abstract void updateSpeed();

    public abstract void overpower();

    public abstract boolean isValid();

    public byte getRunningState() {
        return this.runningState;
    }

    public void setRunningState(int i) {
        boolean oldRun = this.func_145831_w().func_180495_p(this.field_174879_c).func_177229_b(BlockMillGenerator.ISACTIVE);
        boolean newRun = oldRun;
        this.runningState = (byte) i;
        if (runningState > 0)
            newRun = true;
        else if (runningState == 0)
            newRun = false;
        if (newRun != oldRun) {
            this.func_145831_w().func_175656_a(this.field_174879_c, this.func_145831_w().func_180495_p(field_174879_c).func_177226_a(BlockMillGenerator.ISACTIVE, newRun));
            func_145831_w().func_180497_b(field_174879_c, this.func_145838_q(), this.func_145838_q().func_149738_a(func_145831_w()), 5);//this.getWorld().markBlockForUpdate(pos);
        }
    }

    public boolean verifyIntegrity() {
        return true;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    //Unless you increase this, expect to see the TESR to pop in as you approach.
    @Override
    @SideOnly(Side.CLIENT)
    public double func_145833_n() {
        return super.func_145833_n() * 3.0D;
    }

    @Override
    public boolean hasCapability(Capability<?> capability, EnumFacing facing) {
        return capability == MechanicalCapability.MECHANICAL_POWER || super.hasCapability(capability, facing);
    }

    @Override
    public <T> T getCapability(Capability<T> capability, EnumFacing facing) {
        if (capability == MechanicalCapability.MECHANICAL_POWER) {
            return MechanicalCapability.MECHANICAL_POWER.cast(this);
        }
        return super.getCapability(capability, facing);
    }

    @Override
    public int getMechanicalOutput(EnumFacing facing) {
        if (func_145838_q() instanceof IMechanical) {
            if (((IMechanical) func_145838_q()).getMechPowerLevelToFacing(func_145831_w(), field_174879_c, facing) > 0)
                return 20;
        }
        return 0;
    }

    @Override
    public int getMechanicalInput(EnumFacing facing) {
        return 0;
    }

    @Override
    public int getMaximumInput(EnumFacing facing) {
        return 0;
    }
}
